
#    Access Server  for  MIDAS Data Base

global env
source [file join $env(MIDASBASE) TclHttpd tcl MIDASDomainHandler.tcl]       ;#  load default support procedures



global MIDAS_MIDASDataBaseAccessServer_Verbose
set MIDAS_MIDASDataBaseAccessServer_Verbose 0

global MIDAS_SOAPDomainHandler_Verbose
set MIDAS_SOAPDomainHandler_Verbose 0

global env

if {[file isdirectory $env(MIDASDataBaseRoot)] == 0} then {
    puts "Warning: MIDAS DataBase root $env(MIDASDataBaseRoot) is invalid or does not exist"
}


# Use namespaces to isolate our methods
namespace eval urn:MIDASDataBaseAccessServer {


    proc AccessServerVerbose {option} {
        global MIDAS_MIDASDataBaseAccessServer_Verbose
        global MIDAS_SOAPDomainHandler_Verbose

        set MIDAS_MIDASDataBaseAccessServer_Verbose $option
#        set MIDAS_SOAPDomainHandler_Verbose $option

        return [rpcvar::rpcvar int 0]
    }

    proc DBnodes {nodeName pattern} {
        set result [do_nodes $nodeName $pattern]
        return [rpcvar::rpcvar string $result]
    }

    proc DBget {nodeName valueName} {
        set result [do_get $nodeName $valueName]
        return [rpcvar::rpcvar string $result]
    }

    proc DBset {nodeName valueName data type} {
        set result [do_set $nodeName $valueName $data $type]
        return [rpcvar::rpcvar int $result]
    }

    proc DBdelete {nodeName valueName} {
        set result [do_delete $nodeName $valueName]
        return [rpcvar::rpcvar int $result]
    }

    proc DBtype {nodeName valueName} {
        set result [do_type $nodeName $valueName]
        return [rpcvar::rpcvar string $result]
    }

    proc DBvalues {nodeName pattern} {
        set result [do_values $nodeName $pattern]
        return [rpcvar::rpcvar string $result]
    }


    # We have to publish the public methods...

     SOAP::export AccessServerVerbose
     SOAP::export nodes
     SOAP::export get
     SOAP::export set
     SOAP::export delete
     SOAP::export type
     SOAP::export values


    ##########    private procedures

    proc do_nodes {nodeName pattern} {
        global env

        set Separator "\\"

#    first split the nodeName into components

        set NM [split $nodeName $Separator]

#    then join to make a file name

        set PATH $env(MIDASDataBaseRoot)
        foreach node $NM {set PATH [file join $PATH $node]}

#    check the nodeName is a directory

        if {[file isdirectory $PATH] == 0} {
             return -code error -errorcode 0x10004 -errorinfo "node is invalid or does not exist"
        }

#    and obtain directories contained

        if {$pattern == ""} then {
            set r [glob -directory $PATH -type d -- *]
        } else {
            set r [glob -directory $PATH -type d -- $pattern]
        }

#    just return the subnode names

        set result {}
        foreach subnode $r {lappend result [file tail $subnode]}

        return $result
    }

    proc do_get {nodeName valueName} {
        global env

        set Separator "\\"

#    first split the nodeName into components

        set NM [split $nodeName $Separator]

#    then join to make a file name

        set PATH $env(MIDASDataBaseRoot)
        foreach node $NM {set PATH [file join $PATH $node]}

#    check the nodeName is a directory

        if {[file isdirectory $PATH] == 0} {
             return -code error -errorcode 0x10004 -errorinfo "node is invalid or does not exist"
        }

#    read the CONTENTS and put into an array

        set z [catch {open [file join $PATH CONTENTS] r} f]
        if {$z != 0} {
            return -code error -errorcode 0x10004 -errorinfo "unable to access CONTENTS for node $PATH: $m"
        }
        set contents "[read $f]"
        close $f

        catch {unset Contents}

        foreach item [split $contents \n] {
            if {$item == ""} {continue}
            set Contents([head $item]) [tail $item]
        }

#    obtain the value

        if {[info exists Contents($valueName)] == 0} {
            return -code error -errorcode 0x10004 -errorinfo "value $valueName does not exist in node $PATH: $m"
        }

        set vl $Contents($valueName)

        return [tail $vl]
    }

    proc do_set {nodeName valueName data type} {
        global env

        set Separator "\\"

#    first split the nodeName into components

        set NM [split $nodeName $Separator]

#    then join to make a file name

        set PATH $env(MIDASDataBaseRoot)
        foreach node $NM {set PATH [file join $PATH $node]}

#    check nodeName is valid and create nodeName if necessary

        if {[file exists $PATH] == 0} {

#    nodeName does not exist  - create it

            set z [catch {file mkdir $PATH} m]
            if {$z != 0} {
                 return -code error -errorcode 0x10004 -errorinfo "unable to create node $PATH: $m"
            }
        } else {

#    nodeName exists - check it is a directory

            if {[file isdirectory $PATH] == 0} {
                return -code error -errorcode 0x10004 -errorinfo "nodeName is invalid"
            }
        }

#    check for the node CONTENTS file and create it if needed

        set z [catch {open [file join $PATH CONTENTS] a} f]
        if {$z != 0} {
            return -code error -errorcode 0x10004 -errorinfo "unable to access CONTENTS for node $PATH: $m"
        }
        close $f

        if {$valueName == ""} then {return 0}    ;#  just create the node

#    read the CONTENTS and put into an array

        set z [catch {open [file join $PATH CONTENTS] r} f]
        if {$z != 0} {
            return -code error -errorcode 0x10004 -errorinfo "unable to access CONTENTS for node $PATH: $m"
        }
        set contents "[read $f]"
        close $f

        catch {unset Contents}

        foreach item [split $contents \n] {
            if {$item == ""} {continue}
            set Contents([head $item]) [tail $item]
        }

#    create or replace the item valueName

        switch $type {
            string -
            long -
            short -
            binary   {set T $type}
            default  {set T string}
        }

        set Contents($valueName) "$T $data"

#    finally rewrite the CONTENTS

        set z [catch {open [file join $PATH CONTENTS] w} f]
        if {$z != 0} {
            return -code error -errorcode 0x10004 -errorinfo "unable to access CONTENTS for node $PATH: $m"
        }

        foreach item [array names Contents] {
             puts $f  "$item $Contents($item)"
        }

        close $f

        return 0
    }

    proc do_delete {nodeName valueName} {
        global env

        set Separator "\\"

#    first split the nodeName into components

        set NM [split $nodeName $Separator]

#    then join to make a file name

        set PATH $env(MIDASDataBaseRoot)
        foreach node $NM {set PATH [file join $PATH $node]}

#    check the nodeName is a directory

        if {[file isdirectory $PATH] == 0} {
             return -code error -errorcode 0x10004 -errorinfo "node is invalid or does not exist"
        }

        if {$valueName == ""} {

#    delete the node and all subnodes

            set z [catch {file delete -force $PATH} m]
            if {$z != 0} {
                 return -code error -errorcode 0x10004 -errorinfo "unable to delete node $PATH: $m"
            }
        } else {

#    just delete the value

#    read the CONTENTS and put into an array

           set z [catch {open [file join $PATH CONTENTS] r} f]
           if {$z != 0} {
               return -code error -errorcode 0x10004 -errorinfo "unable to access CONTENTS for node $PATH: $m"
           }
           set contents "[read $f]"
           close $f

           catch {unset Contents}
   
           foreach item [split $contents \n] {
               if {$item == ""} {continue}
               set Contents([head $item]) [tail $item]
           }

           catch {unset Contents($valueName)}    ;# remove the value

#    finally rewrite the CONTENTS

           set z [catch {open [file join $PATH CONTENTS] w} f]
           if {$z != 0} {
               return -code error -errorcode 0x10004 -errorinfo "unable to access CONTENTS for node $PATH: $m"
           }

           foreach item [array names Contents] {
                puts $f  "$item $Contents($item)"
           }

           close $f
        }

        return 0
    }

    proc do_type {nodeName valueName} {
        global env

        set Separator "\\"

#    first split the nodeName into components

        set NM [split $nodeName $Separator]

#    then join to make a file name

        set PATH $env(MIDASDataBaseRoot)
        foreach node $NM {set PATH [file join $PATH $node]}

#    check the nodeName is a directory

        if {[file isdirectory $PATH] == 0} {
             return -code error -errorcode 0x10004 -errorinfo "node is invalid or does not exist"
        }

#    read the CONTENTS and put into an array

        set z [catch {open [file join $PATH CONTENTS] r} f]
        if {$z != 0} {
            return -code error -errorcode 0x10004 -errorinfo "unable to access CONTENTS for node $PATH: $m"
        }
        set contents "[read $f]"
        close $f

        catch {unset Contents}

        foreach item [split $contents \n] {
            if {$item == ""} {continue}
            set Contents([head $item]) [tail $item]
        }

#    obtain the type

        if {[info exists Contents($valueName)] == 0} {
            return -code error -errorcode 0x10004 -errorinfo "value $valueName does not exist in node $PATH: $m"
        }

        set vl $Contents($valueName)

        return [head $vl]
    }

    proc do_values {nodeName pattern} {
        global env

        set Separator "\\"

#    first split the nodeName into components

        set NM [split $nodeName $Separator]

#    then join to make a file name

        set PATH $env(MIDASDataBaseRoot)
        foreach node $NM {set PATH [file join $PATH $node]}

#    check the nodeName is a directory

        if {[file isdirectory $PATH] == 0} {
             return -code error -errorcode 0x10004 -errorinfo "node is invalid or does not exist"
        }

#    read the CONTENTS and put into an array

        set z [catch {open [file join $PATH CONTENTS] r} f]
        if {$z != 0} {
            return -code error -errorcode 0x10004 -errorinfo "unable to access CONTENTS for node $PATH: $m"
        }
        set contents "[read $f]"
        close $f

        catch {unset Contents}

        foreach item [split $contents \n] {
            if {$item == ""} {continue}
            set Contents([head $item]) [tail $item]
        }

#    return all matching values

        if {$pattern == ""} then {set pat *} else {set pat $pattern}

        set result {}

        foreach item [array names Contents] {
             if {[string match $pat $item] == 1} then {lappend result $item}
        }

        return $result
    }


     
}   ;#  end of    namespace eval urn:MIDASDataBaseAccessServer

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /MIDASDataBaseAccessServer -namespace urn:MIDASDataBaseAccessServer -uri urn:MIDASDataBaseAccessServer]


proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/MIDASDataBaseAccessServer {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}

