
#   MIDAS Server for the SigTask service

global env
source [file join $env(MIDASBASE) TclHttpd tcl MIDASDomainHandler.tcl]      ;#  load default support proceduresdebug

global MIDAS_SigTaskService_Verbose
set MIDAS_SigTaskService_Verbose 0

global MIDAS_SOAPDomainHandler_Verbose
set MIDAS_SOAPDomainHandler_Verbose 0

global MIDAS_RegisterAccessServer_Verbose
set MIDAS_RegisterAccessServer_Verbose 0

global tcl_platform
if {$tcl_platform(os) != "Windows NT"} then {package require Tclx}  ;#  for kill (signal)



# Use namespaces to isolate your methods
namespace eval urn:SigTaskService {

global env
source [file join $env(MIDASBASE) TclHttpd tcl RegisterAccessServer.tcl]    ;#  load common procedures

    set sigName1 [list "NONE" "SIGHUP"  "SIGINT"  "SIGQUIT"   "SIGILL"     "SIGTRAP"  "SIGBRK"  "SIGIOT"  \
          "SIGABRT" "SIGCORE" "SIGEMT"  "SIGFPE"  "SIGKILL"   "SIGBUS"     "SIGSEGV"  "SIGSYS"  "SIGPIPE" \
          "SIGALRM" "SIGTERM" "SIGURG"  "SIGSTOP" "SIGTSTP"   "SIGCONT"    "SIGCHLD"  "SIGCLD"  "SIGTTIN" \
          "SIGTTOU" "SIGIO"   "SIGXCPU" "SIGXFSZ" "SIGVTALRM" "SIGPROF"    "SIGWINCH" "SIGLOST"           \
          "SIGUSR1" "SIGUSR2"  "SIGPRIO" "SIGPOLL" "SIGPWR"    "SIGWAITING" "SIGLWP"   "SIGFREEZE"         \
          "SIGTHAW" "SIGCANCEL"]
          
    set sigName2 [list "NONE"  "HUP"  "INT"    "QUIT"    "ILL"   "TRAP" "BRK"  "IOT"  \
          "ABRT" "CORE" "EMT"  "FPE"  "KILL"   "BUS"     "SEGV"  "SYS"  "PIPE"        \
          "ALRM" "TERM" "URG"  "STOP" "TSTP"   "CONT"    "CHLD"  "CLD"  "TTIN"        \
          "TTOU" "IO"   "XCPU" "XFSZ" "VTALRM" "PROF"    "WINCH" "LOST"               \
          "USR1" "USR2"  "PRIO" "POLL" "PWR"    "WAITING" "LWP"   "FREEZE"             \
          "THAW" "CANCEL"]

    set RegisterIndex 1

#    SigTask register attributes
#
#    -t         = PID
#    -s         = signal    (number or name)
#    -m         = mode  (Windows only)    =0 use Win32 API;  =1 use cygwin API

    set SigTask_options {
          {t.arg 0  "PID"}
          {s.arg 0  "signal"}
          {m.arg 0  "mode"}
    }

    set SigTask_Updateoptions {
          {t.arg 0xa55a  "PID"}
          {s.arg 0xa55a  "signal"}
          {m.arg 0xa55a  "mode"}
    }

    set SigTask_Usage ": SigTask  -t ? -s ? -m 0|1"

    proc ServerVerbose {option} {
        global MIDAS_SigTaskService_Verbose
        global MIDAS_RegisterAccessServer_Verbose
        global MIDAS_SOAPDomainHandler_Verbose
        set MIDAS_SOAPDomainHandler_Verbose $option
        set MIDAS_RegisterAccessServer_Verbose $option
        set MIDAS_SigTaskService_Verbose $option
        return [rpcvar::rpcvar string "0 OK"]
    }

    proc DefineRegister {Register Class Attributes} {
        global MIDAS_SigTaskService_Verbose
        upvar RegisterIndex RegisterIndex
        upvar RegisterNames RegisterNames

        if {$MIDAS_SigTaskService_Verbose} \
             {puts "called method SigTaskService::DefineRegister with parameters $Register $Class $Attributes"}
             
#    Check Register name is not already in use

        if {[info exists RegisterNames($Register)]} then {
             return -code error -errorcode 0x20006 -errorinfo "Register name already in use"
        }

#    Check Class is valid

        switch $Class {
            sigTask -
            SigTask  {}

            default {
               return -code error -errorcode 0x20007 -errorinfo "Class name is invalid or not supported"
            }
        }
        
#    create register

        switch $Class {
            sigTask -
            SigTask  {

                set RegisterNames($Register) $RegisterIndex

                upvar R$RegisterIndex R
                set R(class) SigTask

                upvar SigTask_options options

                foreach item $options {
                    set opt [first [split [first $item] .]]
                    set R($opt) [second $item]
                }

                incr RegisterIndex
            }
            default {
                return -code error -errorcode 0x20007 -errorinfo "Class name is invalid or not supported"
            }
        }

        
#    apply attributes

        return [do_WriteRegisterAttributes $Register $Attributes]
    }

    proc do_ReadRegisterAttributes {Register} {

        global MIDAS_SigTaskService_Verbose
        upvar 2 RegisterNames RegisterNames

        set Index $RegisterNames($Register)
        upvar 2 R$Index R

        if {$MIDAS_SigTaskService_Verbose} \
             {puts "called method SigTaskService::ReadRegisterAttributes with parameter $Register"}

        switch $R(class) {
            SigTask {
                 set attributes "-t$R(t) -s$R(s) -m$R(m)"
            }
            default {
                return "0x2000e Invalid request"
            }
        }

        return $attributes
    }
    
    proc do_WriteRegisterAttributes {Register Attributes} {
        upvar 2 sigName1 sigName1
        upvar 2 sigName2 sigName2
    
        global MIDAS_SigTaskService_Verbose
        upvar 2 RegisterNames RegisterNames

        set Index $RegisterNames($Register)
        upvar 2 R$Index R

        if {$MIDAS_SigTaskService_Verbose} \
             {puts "called method SigTaskService::WriteRegisterAttributes with parameters $Register $Attributes"}

        switch $R(class) {
            SigTask  {

                upvar 2 SigTask_Updateoptions options
                upvar 2 SigTask_Usage usage

                array set params [::cmdline::getKnownOptions Attributes $options $usage]

                foreach m [array names params] {
                    if {$params($m) == 0xa55a} {continue}
                    switch $m {
                        t  {set R(t) $params(t)}
                        s  {
                           if {[scan $params(s) %i vl] != 0} then {
                               set R(s) $params(s)    ;#    value is a number 
                           } else {
                               set i [lsearch $sigName1 $params(s)]
                               if {$i > 0} then {
                                   set R(s) $params(s)       ;#     value is a signal name of the form SIGxxx
                               } else {
                                   set i [lsearch $sigName2 $params(s)]
                                   if {$i > 0} then {
                                       set R(s) $params(s)  ;#     value is a signal name of the form xxx
                                   }
                               }
                           }
                        }
                        m  {set R(m) $params(m)}
                        default {}
                    }
                }
            }
            default {
                return -code error -errorcode 0x10004 -errorinfo "Invalid request"
            }
        }
        return 0
    }

    proc do_WriteRegister {Register Value} {

        global MIDAS_SigTaskService_Verbose
        upvar 2 RegisterNames RegisterNames

        set Index $RegisterNames($Register)
        upvar 2 R$Index R

        if {$MIDAS_SigTaskService_Verbose} \
             {puts "called method SigTaskService::WriteRegister with parameters $Register $Value"}

        switch $R(class) {
            SigTask  {
#                puts "Sig request t=$R(t) s=$R(s) m=$R(m)"

                global tcl_platform
                if {$tcl_platform(os) != "Windows NT"} then {
                    kill $R(s) $R(t)
                } else {
                    set z [catch {exec kill -SIG$R(s) $R(t)} m]
                    if {$z != 0} {puts "Sig request t=$R(t) s=$R(s) m=$R(m); kill -SIG$R(s) $R(t); returned $m"}
                }
            }
            default {
                return -code error -errorcode 0x10004 -errorinfo "Invalid request"
            }
        }
 
        return 0
    }

    proc do_ReadRegister {Register} {

        global MIDAS_SigTaskService_Verbose
        upvar 2 RegisterNames RegisterNames

        set Index $RegisterNames($Register)
        upvar 2 R$Index R

        if {$MIDAS_SigTaskService_Verbose} \
             {puts "called method SigTaskService::ReadRegister with parameters $Register"}

        switch $R(class) {
            SigTask  {
#      There is no read function  -  treate as NOOP
            }
            default {
                return -code error -errorcode 0x10004 -errorinfo "Invalid request"
            }
        }
        return 0
    }

    # We have to publish the public methods...

     SOAP::export ServerVerbose
     SOAP::export DefineRegister
     SOAP::export InquireRegisters
     SOAP::export ReadRegisterAttributes
     SOAP::export ReadRegistersAttributes
     SOAP::export DeleteRegister
     SOAP::export DeleteRegisters
     SOAP::export WriteRegisterAttributes
     SOAP::export WriteRegistersAttributes
     SOAP::export WriteRegister
     SOAP::export WriteRegisters
     SOAP::export InitialiseRegister
     SOAP::export InitialiseRegisters
     SOAP::export ReadRegister
     SOAP::export ReadRegisters
     
}

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /SigTaskService -namespace urn:SigTaskService -uri urn:SigTaskService]


proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/SigTaskService {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}



